@@ -125,6 +125,13 @@ module Nokogiri
         @xml.root.parse('<hello>')
         assert(error_count < @xml.errors.length, "errors should have increased")
       end
+      
+      def test_parse_error_on_fragment_with_empty_document
+        doc = Document.new
+        fragment = DocumentFragment.new(doc, '<foo><bar/></foo>')
+        node = fragment%'bar'
+        node.parse('<baz><</baz>')
+      end
 
       def test_subclass_dup
         subclass = Class.new(Nokogiri::XML::Node)
@@ -209,6 +216,13 @@ module Nokogiri
         end
       end
 
+      def test_append_with_attr
+        r = Nokogiri.XML('<r a="1" />').root
+        assert_raises(ArgumentError) do
+          r << r.at_xpath('@a')
+        end
+      end
+
       def test_inspect_ns
         xml = Nokogiri::XML(<<-eoxml) { |c| c.noblanks }
           <root xmlns="http://tenderlovemaking.com/" xmlns:foo="bar">
@@ -440,6 +454,11 @@ module Nokogiri
         assert_equal node, @xml.xpath('//address').first
       end
 
+      def test_at_self
+        node = @xml.at('address')
+        assert_equal node, node.at('.')
+      end
+
       def test_at_xpath
         node = @xml.at_xpath('//address')
         nodes = @xml.xpath('//address')
@@ -592,6 +611,19 @@ module Nokogiri
         assert_nil address['domestic']
       end
 
+      def test_attribute_setter_accepts_non_string
+        address = @xml.xpath("/staff/employee/address").first
+        assert_equal "Yes", address[:domestic]
+        address[:domestic] = "Altered Yes"
+        assert_equal "Altered Yes", address[:domestic]
+      end
+
+      def test_attribute_accessor_accepts_non_string
+        address = @xml.xpath("/staff/employee/address").first
+        assert_equal "Yes", address["domestic"]
+        assert_equal "Yes", address[:domestic]
+      end
+
       def test_delete
         address = @xml.xpath('/staff/employee/address').first
         assert_equal 'Yes', address['domestic']
@@ -713,6 +745,11 @@ module Nokogiri
         assert_equal node.parent, parent_node
       end
 
+      def test_search_self
+        node = @xml.at('//employee')
+        assert_equal node.search('.').to_a, [node]
+      end
+
       def test_search_by_symbol
         assert set = @xml.search(:employee)
         assert 5, set.length
@@ -807,6 +844,46 @@ module Nokogiri
         assert_equal 1, tires.length
       end
 
+      def test_namespaced_attribute_search_with_xpath
+        # from #593
+        xmlContent = <<-EOXML
+<?xml version="1.0"?>
+<ns1:el1 xmlns:ns1="http://blabla.com" >
+  <ns1:el2 ns1:att="123">with namespace</ns1:el2 >
+  <ns1:el2 att="noNameSpace">no namespace</ns1:el2 >
+</ns1:el1>
+EOXML
+        xml_doc = Nokogiri::XML(xmlContent)
+
+        no_ns = xml_doc.xpath("//*[@att]")
+        assert_equal no_ns.length, 1
+        assert_equal no_ns.first.content, "no namespace"
+
+        with_ns = xml_doc.xpath("//*[@ns1:att]")
+        assert_equal with_ns.length, 1
+        assert_equal with_ns.first.content, "with namespace"
+      end
+
+      def test_namespaced_attribute_search_with_css
+        # from #593
+        xmlContent = <<-EOXML
+<?xml version="1.0"?>
+<ns1:el1 xmlns:ns1="http://blabla.com" >
+  <ns1:el2 ns1:att="123">with namespace</ns1:el2 >
+  <ns1:el2 att="noNameSpace">no namespace</ns1:el2 >
+</ns1:el1>
+EOXML
+        xml_doc = Nokogiri::XML(xmlContent)
+
+        no_ns = xml_doc.css('*[att]')
+        assert_equal no_ns.length, 1
+        assert_equal no_ns.first.content, "no namespace"
+
+        with_ns = xml_doc.css('*[ns1|att]')
+        assert_equal with_ns.length, 1
+        assert_equal with_ns.first.content, "with namespace"
+      end
+
       def test_namespaces_should_include_all_namespace_definitions
         xml = Nokogiri::XML.parse(<<-EOF)
         <x xmlns="http://quux.com/" xmlns:a="http://foo.com/" xmlns:b="http://bar.com/">
@@ -854,7 +931,7 @@ module Nokogiri
             <b:div>hello b</b:div>
             <c:div>hello c</c:div>
             <div>hello moon</div>
-          </y>  
+          </y>
         </x>
         EOF
         set = xml.search("//y/*")
@@ -912,6 +989,41 @@ module Nokogiri
         assert_equal @xml, node_set.document
         assert node_set.respond_to?(:awesome!)
       end
+
+      def test_blank
+        doc = Nokogiri::XML('')
+        assert_equal false, doc.blank?
+      end
+
+      def test_to_xml_allows_to_serialize_with_as_xml_save_option
+        xml = Nokogiri::XML("<root><ul><li>Hello world</li></ul></root>")
+        set = xml.search("//ul")
+        node = set.first
+
+        assert_no_match("<ul>\n  <li>", xml.to_xml(:save_with => XML::Node::SaveOptions::AS_XML))
+        assert_no_match("<ul>\n  <li>", node.to_xml(:save_with => XML::Node::SaveOptions::AS_XML))
+      end
+
+      # issue 647
+      def test_default_namespace_should_be_created
+        subject = Nokogiri::XML.parse('<foo xml:bar="http://bar.com"/>').root
+        ns = subject.attributes['bar'].namespace
+        assert_not_nil ns
+        assert_equal ns.class, Nokogiri::XML::Namespace
+        assert_equal 'xml', ns.prefix
+        assert_equal "http://www.w3.org/XML/1998/namespace", ns.href
+      end
+
+      # issue 648
+      def test_namespace_without_prefix_should_be_set
+        node = Nokogiri::XML.parse('<foo xmlns="http://bar.com"/>').root
+        subject = Nokogiri::XML::Node.new 'foo', node.document
+        subject.namespace = node.namespace
+        ns = subject.namespace
+        assert_equal ns.class, Nokogiri::XML::Namespace
+        assert_nil ns.prefix
+        assert_equal ns.href, "http://bar.com"
+      end
     end
   end
 end
